//=======================================================================
//
//  WordGen.java
//
//  Description:  Generates words using 
//
// Generates words using the Traveller word generation rules.  The language
// rules are declared in a file structured like a Windows INI file; that is,
// divided into sections headed by a [HEADER] line, followed by a series of
// key=value lines.  See WORDGEN.TXT for more information.
//
// Based on the Wordgen source code in Euphoria by 
//             Jeff Zeitlin (jeff.zeitlin@execnet.com)
// 
// To do ? : Change to use .properties or XML language source files instead of 
//           non-standard Windows format ?
//         : Optimise read and generation 
//         :
// 
//=======================================================================
//  Copyright (c) Frank G. Pitt, 1 June 2000
//
//  Permission is hereby granted to use or modify this code for 
//  non-commercial purposes as long as the above copyright is 
//  retained or reproduced in any derivative work. 
//
//=======================================================================
//                         Maintenance Log
//------------|-------------|--------------------------------------------
// dd/mm/yy   | Author      |                 Description
//------------|-------------|--------------------------------------------
// 10/08/2000 | Frank Pitt  | Created
//------------|-------------|--------------------------------------------
// ??/??/2000 |             | 
//------------|-------------|--------------------------------------------
//   Record your modifications !
//=======================================================================
// package gen.mundens.rpg.language

import java.util.* ;
import java.io.* ;
              
class WordGen 
{
  public WordGen ( String language, int numWords, String outfile )
  {  
    this.language = language; 
    this.numWords = numWords; 
    this.outfile  = outfile; 
  }

  public void execute() 
  {
    Language lang = new Language ( this.language ) ;
    //  Note that unlike the Euphoria implementation,
    //  this version uses a sortedSet, so does not need 
    //  to sort and clean the list at the end because 
    //  we should never have duplicates, and it will be 
    //  sorted as entries are added.
    //
    //  If we try to add a duplicate, it will fail, so
    //  we don't break out of the loop until we 
    //  have generated the require number of words    

    wordSet = new TreeSet() ;

    while ( wordSet.size() < numWords )
    {
       wordSet.add( lang.makeWord() );
    }
    writeWords() ;
  }

  private void writeWords()
  { 
    PrintStream outstr = System.out ;    // default to stdout

     
    if ( this.outfile != null )
    {
      try
      {
        outstr =  new PrintStream( new FileOutputStream ( this.outfile ) ) ;
      }
      catch( FileNotFoundException fnfe )
      {
        System.out.println( " output file unable to be created." ) ;
      } 
    }

    Iterator iter = wordSet.iterator() ;

    while ( iter.hasNext() )
    {
       outstr.println( ( (String) iter.next() ) ) ;
    }
  } 

  public static void main ( String args[] )
  {
     // default parameters, load from properties file later. 
     String language = "Wordgen" ;
     int    numWords = 100 ;
     
     // sorry, Jeff, I hate default output files, 
     // all good programs should defult to stdout 
     // so they can be properly piped  !
     String outfile = null ; 

     int cur = 0 ;

     while ( cur < args.length )
     {
       switch ( args[cur].charAt(1) )
       {
         case 'L' :
                   language = args[ cur + 1 ] ;
                   break ;
         case 'N' :
                   try
                   {
                     numWords = Integer.parseInt( args[ cur + 1 ] ) ;
                   }
                   catch ( NumberFormatException nfe ) 
                   {
                    System.out.println( "Invalid number of words  specified" ) ;
                    System.exit(1) ;
                   }
                   break ;
         case 'O' :
                   outfile  = args[ cur + 1 ] ;
                   break ;
       }
       cur += 2 ;
     }

     WordGen wg = new WordGen( language, numWords, outfile) ;
     wg.execute() ;
  } 

  // private attributes  
  private String  language ;
  private int     numWords ; 
  private String  outfile  ;
  private TreeSet wordSet  ;

}


